/*
 * @psiFrame.java Version 1.1 5/27/2016
 *
 * Copyright (c) 2016 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package psi.gui;

import java.sql.Timestamp;
import java.sql.Date;
import java.text.SimpleDateFormat;
import java.text.ParseException;

import java.awt.*;
import java.awt.event.*;
import java.util.Observer;
import java.util.Observable;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

import cmn.cmnStruct;			         // Global Common Data Structure

import iqstrat.iqstratHeadersListStruct; // Well Header List Structure
import iqstrat.iqstratHeadersStruct;     // Well Header Data Structure
import iqstrat.io.ReadHeadersXMLFile;    // Read Well Header XML Stream

import iqstrat.iqstratFilesListStruct;   // File Location List Structure
import iqstrat.iqstratFilesStruct;       // File Location Data Structure
import iqstrat.io.ReadFilesXMLFile;      // Read Pressure File Location Data

import quake.quakeDataListStruct;        // Quake Data List Structure
import quake.quakeSeisStruct;            // Quake Seismic Data Structure
import quake.io.ReadQuakeXMLFile;        // Read Quake XML Stream from KGS DB

import quake.quakeSensorsStruct;         // Sensor Data Structure
import quake.io.ReadSensorXMLFile;       // Read Sensor Location Data from DB

import quake.gui.quakeTablePanel;        // Seismic Events Catalog Panel

import seismic.seisStruct;               // Seismic Data Structure
import seismic.io.seisReadMSeed;         // Seismic Read & Parse miniseed File

import psi.io.psiCSVFile;                // Pressure Read & Parse CSV File
import psi.gui.psiControlFrame;          // Pressure-Seismic Plot Control Frame

/** Class psiFrame
 *  <p> This Class will allow the user to select the status of the data
 *
 *  @version 1.1 5/27/2016
 *  @author  John R. Victorine
 */

public class psiFrame extends JFrame implements ActionListener, Observer
{
  // Input Variables

  private cmnStruct              stCMN      = null;  // Common Data Structure

  // Global Variables

  private Observable             notifier   = null; // Observable notifier

  private quakeTablePanel        pCatalog   = null; // Seismic Catalog Panel

  private iqstratHeadersStruct   stWell     = null; // headers Information Data Structure
  private quakeSeisStruct        stCatalog  = null; // Seismic Data Catalog
  private quakeSensorsStruct     stSensor   = null; // Seismic Sensor Data Location
  private iqstratFilesListStruct stPSI      = null; // Pressure Data Files Location
  private iqstratFilesListStruct stMiniSeed = null; // miniSeed Data Files Location

  private int                    iUTC_hrs   = 0;    // Conversion from Central to UCT

  private Timestamp              tsStart    = null; // Starting Date
  private Timestamp              tsEnd      = null; // Ending Date

  private int                    iSensor    = -1;
  private int                    iChannel   = -1;

  private psiControlFrame        control    = null; // Pressure-Seismic Plot Control Frame
  private seisStruct             stWave     = null; // Seismic Data Structure
  private seisStruct             stSeis     = null; // Seismic Data Structure with Pressure Data
  private int                    iEvent     = -1;

  // -- Plot Screen Location & Size

  private int iFrameX       = -1;   // Plot Frame X-Coordintate
  private int iFrameY       = -1;   // Plot Frame Y-Coordintate
  private int iFrameWidth   = -1;   // Plot Frame Width
  private int iFrameHeight  = -1;   // Plot Frame Height

  // -- Screen Width

  private int iControlWidth = 425;//550;  // Control Panel Width
  private int iScreenWidth  = 0;    // Screen Width
  private int iScreenHigh   = 0;    // Scree Hieght

  // Global Frame Widgets

  private JButton      btnSelect = new JButton();
  private JButton      btnCancel = new JButton();

  private int          irb       = 0;
  private JRadioButton rb[]      = null;

  private int          irbC      = 3;
  private String       srbC[]    = { "EHZ", "EHN", "EHE" };
  private JRadioButton rbC[]     = null;

  /** Constructor psiFrame()
   *  <p> This is the Constructor for this class.
   *  @param stCMN      = Common Data Structure
   */

  public psiFrame( cmnStruct stCMN )
  {
    try
    {
      this.stCMN = stCMN;

      readWell();
	  readSeismicCatalog();
	  readSeismicSensors();
      readPressureFilesLocation();
      readMiniSeedFilesLocation();

      long      lstart  = (long) 0;
      long      lend    = (long) 0;
      long      lmin    = (long) 0;
      long      lmax    = (long) 0;

      if (stSensor != null)
        irb = stSensor.iTotal;

      if (stPSI != null)
      {
		for (int i=0; i<stPSI.iCount; i++)
		{
  		  lstart = stPSI.stItem[i].tsStart.getTime();
		  lend   = stPSI.stItem[i].tsEnd.getTime();

		  if (i==0)
		  {
			lmin = stPSI.stItem[i].tsStart.getTime();
			lmax = stPSI.stItem[i].tsEnd.getTime();
		  }

		  if (lmin > lstart)
		    lmin = lstart;

		  if (lmax < lend)
		    lmax = lend;
		}

		tsStart = new Timestamp(lmin);
		tsEnd   = new Timestamp(lmax);
System.out.println(tsStart.toString()+" "+tsEnd.toString());
	  }

      jbInit();
      pCatalog.setParent(this);

      addWindowListener( new psiFrame_WindowListener() );
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Frame for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
    JPanel pnlButtons  = new JPanel();
    JPanel pnl         = new JPanel();
    JPanel pnlLoad     = new JPanel();
    JScrollPane scroll = new JScrollPane();

    JTabbedPane tabbedPane   = new JTabbedPane();

    // Create an Observable

    notifier = new psiFrameObservable();

    // Add this Dialog to the notifier as observer

    notifier.addObserver(this);

    // Build Frame

    this.setTitle("Pressure/Seismic Control Dialog");
    this.getContentPane().setLayout(new BorderLayout());

    pnlButtons.setBorder(BorderFactory.createEtchedBorder());

    btnSelect.setFont(new java.awt.Font("Dialog", 1, 11));
    btnSelect.setText("Select");
    btnSelect.addActionListener(this);

    btnCancel.setFont(new java.awt.Font("Dialog", 1, 11));
    btnCancel.setText("Cancel");
    btnCancel.addActionListener(this);

    pnlLoad = buildLoadPanel();

//    pnl.setLayout(new BorderLayout());

//    pTable = new quakeTable(st, tsStart, tsEnd);
//    scroll = pTable.getScrollPane();

    // Attach Widgets to frame

//    this.getContentPane().add(pnlButtons, BorderLayout.SOUTH);
    pnlButtons.add(btnSelect,      null);
    pnlButtons.add(btnCancel,      null);

    this.getContentPane().add(tabbedPane,  BorderLayout.CENTER);
    tabbedPane.add(pnlLoad,  "Load Data - By Seismic Event");

//    this.getContentPane().add(pnl, BorderLayout.CENTER);
//    pnl.add(scroll,                BorderLayout.CENTER);

    // Display the Frame

    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    iScreenWidth = d.width;
    iScreenHigh  = d.height-50;

    this.setSize(new Dimension(iControlWidth, iScreenHigh));
    this.setLocation(iScreenWidth-iControlWidth-25,1);
//    this.setLocation((d.width - this.getSize().width) / 2,
//                     (d.height - this.getSize().height) / 2);

    this.setResizable(true);
    this.setVisible(true);
  }

  /** Method buildLoadPanel()
   *  <p> This method will build the Load Data Panel
   *  @return panel = this panel
   */

  private JPanel buildLoadPanel()
  {
	JPanel panel       = new JPanel();
	JPanel pnlTop      = new JPanel();
	JPanel pnlSensors  = new JPanel();
	JPanel pnlChannels = new JPanel();

    ButtonGroup group  = new ButtonGroup();
    ButtonGroup groupC = new ButtonGroup();

    TitledBorder titledBorderSensors = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "Active Seismic Sensors:");
    titledBorderSensors.setTitleFont(new java.awt.Font("Dialog", 1, 12));

    TitledBorder titledBorderChannels = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "Channels:");
    titledBorderChannels.setTitleFont(new java.awt.Font("Dialog", 1, 12));

    panel.setLayout(new BorderLayout());

    // Seismic Catalog Panel

    pCatalog = new quakeTablePanel( notifier, stCatalog, tsStart, tsEnd);

    // Seismic Sensors Panel

    pnlTop.setLayout(new BorderLayout());

    // -- Build Sensor Panel

    pnlSensors.setLayout(new GridLayout(3,3));
    pnlSensors.setBorder(titledBorderSensors);

    rb = new JRadioButton[irb];

    for (int i=0; i<irb; i++)
    {
      rb[i] = new JRadioButton();
      if (stSensor != null)
      {
        if (stSensor.sName[i].equals("WK12"))
        {
		  rb[i].setSelected(true);
		  iSensor = i;
		}
        rb[i].setText( stSensor.sName[i] );
	  }
	  rb[i].addActionListener(this);

      group.add(rb[i]);

      pnlSensors.add(rb[i], null);
	}

    // -- Build Channel Panel

    pnlChannels.setLayout(new GridLayout());
    pnlChannels.setBorder(titledBorderChannels);

    rbC = new JRadioButton[irbC];

    for (int i=0; i<irbC; i++)
    {
      rbC[i] = new JRadioButton();
      if (i==0)
      {
		rbC[i].setSelected(true);
		iChannel = i+1;
	  }
      rbC[i].setText( srbC[i] );
	  rbC[i].addActionListener(this);

      groupC.add(rbC[i]);

      pnlChannels.add(rbC[i], null);
	}

    // Attach Widgets to Panel

    panel.add(pnlTop,       BorderLayout.NORTH);
    pnlTop.add(pnlSensors,  BorderLayout.NORTH);
    pnlTop.add(pnlChannels, BorderLayout.SOUTH);

    panel.add(pCatalog,     BorderLayout.CENTER);

	return (panel);
  }

  /* ===================================================================== *
   * ----------------------------- CLOSE --------------------------------- *
   * ===================================================================== */

  /** METHOD psiFrame_WindowListener()
   *  <p> This Class will handle Frame Close Action using the X Button on Frame
   */

  public class psiFrame_WindowListener extends WindowAdapter
  {
    public void windowClosing(WindowEvent e) { close(); }
  }

  /** Method close()
   * <p> This method will close this dialog and all other dialogs opened by
   *     this dialog
   */

  public void close()
  {
	stCMN        = null;
    notifier     = null; // Observable notifier

    tsStart      = null; // Starting Timestamp
    tsEnd        = null; // Ending Timestamp

    if (stWell != null)
      stWell.delete();
    stWell     = null; // headers Information Data Structure

    if (pCatalog != null)
      pCatalog.close();
    pCatalog     = null;

    if (stCatalog != null)
      stCatalog.delete();
    stCatalog = null;      // Seismic Data Catalog

    if (stSensor != null)
      stSensor.delete();
    stSensor  = null; // Seismic Sensor Data Location

    if (stPSI != null)
      stPSI.delete();
    stPSI = null; // Pressure Data File Location

    if (stMiniSeed != null)
      stMiniSeed.delete();
    stMiniSeed = null; // miniSeed Data File Location

    tsStart = null; // Starting Date
    tsEnd   = null; // Ending Date


	if (control != null)
	  control.close();
	control = null;

    if (stWave != null)
      stWave.delete();
    stWave   = null;  // Seismic Data Structure

    if (stSeis != null)
      stSeis.delete();
    stSeis   = null;  // Seismic Data Structure with Pressure Data


    btnSelect    = null;
    btnCancel    = null;

    if (rb != null)
    {
      for (int i=0; i<irb; i++)
      {
        rb[i] = null;
	  }
    }
    rb = null;

    dispose();
  }

  /* ===================================================================== *
   * --------------------------- READ DATA ------------------------------- *
   * ===================================================================== */

  /** Method readWell()
   * <p> This method will read the Wellington KGS 1-28 well information xml data
   */

  public void readWell()
  {
	iqstratHeadersListStruct st = null;

    ReadHeadersXMLFile pRead = new ReadHeadersXMLFile(iqstrat.io.ReadHeadersXMLFile.URL);
    st = pRead.Process(cmn.cmnStruct.GET_1_28);

    if (st != null)
    {
      if (st.iCount > 0)
	    stWell = iqstrat.iqstratHeadersUtility.copy( st.stItem[0] );
	}
  }

  /** Method readSeismicCatalog()
   * <p> This method will read the Earthquake Data from the KGS DB xml stream
   */

  public void readSeismicCatalog()
  {
    quakeDataListStruct st = null;

  	ReadQuakeXMLFile pRead = new ReadQuakeXMLFile(quake.io.ReadQuakeXMLFile.URL);
    st = pRead.Process( cmn.cmnStruct.EVENTS );

    stCatalog = quake.quakeDataUtility.quakeGetSeismic( st );
//      quake.quakeDataUtility.quakeGetSeismic( quake.quakeDataUtility._CENTRAL, st );
    stCatalog = quake.quakeSeisUtility.bubbleSort(stCatalog);
  }

  /** Method readSeismicSensors()
   *  <p> This method will read the Sensor information from the KGS DB
   */

  private void readSeismicSensors()
  {
    ReadSensorXMLFile pRead = new ReadSensorXMLFile(quake.io.ReadSensorXMLFile.URL);
    stSensor = pRead.Process(cmn.cmnStruct.GET_SENSORS);
  }

  /** Method readPressureFilesLocation()
   *  <p> This method will read the Pressure File information from the KGS DB
   */

  private void readPressureFilesLocation()
  {
    ReadFilesXMLFile pRead = new ReadFilesXMLFile(iqstrat.io.ReadFilesXMLFile.URL);
    stPSI = pRead.Process(cmn.cmnStruct.PRESSURE_FILES);
  }

  /** Method readMiniSeedFilesLocation()
   *  <p> This method will read the miniSeed File information from the KGS DB
   */

  private void readMiniSeedFilesLocation()
  {
    ReadFilesXMLFile pRead = new ReadFilesXMLFile(iqstrat.io.ReadFilesXMLFile.URL);
    stMiniSeed = pRead.Process(cmn.cmnStruct.MINISEED_FILES);
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getSelectedRow()
   *  <p> This method will return the selected row.
   *  @return iSelectedRow - Selected Row.
   */

//  public int getSelectedRow() { return (pTable.getSelectedRow()); }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** Method select()
   * <p> This method will close this dialog and all other dialogs opened by
   *     this dialog
   */

  private void select()
  {
  }

  /** Method getMiniSeedFile
   * <p> This method will construct the file path and filename of the miniSeed file
   *     that will display the seismic event.
   * @param  ts    = the Date of the seismic event
   * @param  sid   = seismic sensor file needed
   * @param  sName = name of sensor
   * @return sFile = URL Path to miniSeed file.
   */

  private String getMiniSeedFile(Timestamp ts, String sid, String sName)
  {
	String sFile  = "";
    int    i = 0;
    String tokens[]    = null;
    String stng        = "";
    String str         = "";
    String sDelimiter  = new String("[ ]+");
    String sDelimiter2 = new String("[:]+");
    String sDate       = "";
    String sTime       = "";
    String sHour       = "";
    String sChannel    = "1";
    String sDay        = "000";

	if ((ts != null) && (sid != null) && (sName != null))
	{
	  stng = new String(ts.toString());

	  if (stng.length() > 0)
	  {
	    str    = new String(stng);
        tokens = str.split( sDelimiter );
        if (tokens.length > 0)
          sDate = new String(tokens[0].trim());

        if (tokens.length > 1)
        {
          sTime = new String(tokens[1].trim());
          tokens = sTime.split( sDelimiter2 );
          if (tokens.length == 3)
          {
			sHour = new String(tokens[0]);
		  }
		}

        sDate = new String(sDate + " 00:00:00.0");
	  }

      if (iChannel > -1)
      {
        sChannel = new String( "" + iChannel);
	  }

      if (stMiniSeed != null)
      {
	    for (i=0; i<stMiniSeed.iCount; i++)
	    {
		  if (sDate.equals(stMiniSeed.stItem[i].tsStart.toString()))
		  {
			if (checkSensor( sName, stMiniSeed.stItem[i] ))
			{
			  sDay  = padDay(new String(""+stMiniSeed.stItem[i].iDay));
		      sFile = new String( stMiniSeed.stItem[i].sURL+
			  	                  stMiniSeed.stItem[i].iYear+"."+
	 	                          sDay+"."+ //stMiniSeed.stItem[i].iDay+"."+
	 	                          sHour+".00.00."+sid+".1."+sChannel+".m" );
			}
			else
			{
              JOptionPane.showMessageDialog((Component) null,
                                             "Sensor " + sName + " data is NOT available for this day",
                                             "WARNING",
                                             JOptionPane.WARNING_MESSAGE);
			}
   	      }
	    }
	  }
	}

	return (sFile);
  }

  /** Method padDay()
   * <p> This method will pad the day to 3 characters.
   * @param  str  = the Day to be converted
   * @return sDay = the day number as a 3 character string
   */

  public String padDay( String str )
  {
    String sDay = "";

    if (str.length() == 1)
      sDay = new String("00" + str);
    else if (str.length() == 2)
      sDay = new String("0" + str);
    else
      sDay = new String(str);

    return (sDay);
  }

  /** Method checkSensor()
   *  <p> This method will check the sensor to make sure that it is available
   *  @param iDay  = the day of the year
   *  @param sName = the sensor
   *  @param st    = Miniseed File Information Data Structure
   */

  private boolean checkSensor( String sName, iqstratFilesStruct st )
  {
	boolean bVal = false;

	if (sName.equals("WK01"))
	{
	  if (st.wk01 == 1) { bVal = true; }
	}

	if (sName.equals("WK02"))
	{
	  if (st.wk02 == 1) { bVal = true; }
	}

	if (sName.equals("WK03"))
	{
	  if (st.wk03 == 1) { bVal = true; }
	}

	if (sName.equals("WK04"))
	{
	  if (st.wk04 == 1) { bVal = true; }
	}

	if (sName.equals("WK05"))
	{
	  if (st.wk05 == 1) { bVal = true; }
	}

	if (sName.equals("WK06"))
	{
	  if (st.wk06 == 1) { bVal = true; }
	}

	if (sName.equals("WK07"))
	{
	  if (st.wk07 == 1) { bVal = true; }
	}

	if (sName.equals("WK08"))
	{
	  if (st.wk08 == 1) { bVal = true; }
	}

	if (sName.equals("WK09"))
	{
	  if (st.wk09 == 1) { bVal = true; }
	}

	if (sName.equals("WK10"))
	{
	  if (st.wk10 == 1) { bVal = true; }
	}

	if (sName.equals("WK11"))
	{
	  if (st.wk11 == 1) { bVal = true; }
	}

	if (sName.equals("WK12"))
	{
	  if (st.wk12 == 1) { bVal = true; }
	}

	if (sName.equals("WK13"))
	{
	  if (st.wk13 == 1) { bVal = true; }
	}

	if (sName.equals("WK14"))
	{
	  if (st.wk14 == 1) { bVal = true; }
	}

	if (sName.equals("WK15"))
	{
	  if (st.wk15 == 1) { bVal = true; }
	}

	return (bVal);
  }

  /** Method getPressureFile
   * <p> This method will construct the file path and filename of the pressure file
   *     that will display the pressure data.
   * @param  ts    = the Date of the seismic event
   * @return sFile = URL Path to Pressure file.
   */

  private iqstratFilesStruct getPressureFile(Timestamp ts)
  {
	iqstratFilesStruct st  = null;
	long   ltime  = (long) 0;
	long   lstart = (long) 0;
	long   lend   = (long) 0;

	if (ts != null)
	{
      ltime  = ts.getTime();

	  if (stPSI != null)
	  {
		for (int i=0; i<stPSI.iCount; i++)
		{
		  lstart = stPSI.stItem[i].tsStart.getTime();
		  lend   = stPSI.stItem[i].tsEnd.getTime();

		  if ((ltime >= lstart) && (ltime < lend))
		  {
			st = stPSI.stItem[i];
		  }
		}
	  }
	}

	return (st);
  }

  /* ===================================================================== *
   * -------------------------- PLOT METHOD ------------------------------ *
   * ===================================================================== */

  /** Method plot()
   *  <p> This method will plot the data
   */

  private void plot()
  {
    Timestamp tsStart = null; // Starting Date
    Timestamp tsEnd   = null; // Ending Date

	if (control != null)
	  control.close();
	control = null;

    if (stWave != null)
    {
      tsStart  = new Timestamp(stWave.tsStart.getTime());
      tsEnd    = new Timestamp(stWave.tsEnd.getTime());
	}
	else
	{
      tsStart  = new Timestamp(stSeis.tsStart.getTime());
      tsEnd    = new Timestamp(stSeis.tsEnd.getTime());
	}

	control = new psiControlFrame( stCMN ,
	                               tsStart,
	                               tsEnd,
	                               stSeis,
	                               stWave,
	                               iUTC_hrs);
	if (iEvent > -1)
	{
	  control.setEvent( stCatalog.ts[iEvent].toString(),
                        stCatalog.tsUTC[iEvent].toString(),
                        stCatalog.mag[iEvent],
                        stCatalog.dLatitude[iEvent],
                        stCatalog.dLongitude[iEvent],
                        stCatalog.depth[iEvent],
                        stCatalog.src[iEvent] );
	}
  }

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    for (int i=0; i<irb; i++)
    {
      if (event.getSource() == rb[i])
      {
		iSensor = i;
	  }
	}

    for (int i=0; i<irbC; i++)
    {
      if (event.getSource() == rbC[i])
      {
	    iChannel = i+1;
	  }
	}

    if (event.getSource() == btnSelect) { select(); }
    if (event.getSource() == btnCancel)
    {
//      if (notifier != null)
//        notifier.notifyObservers( new String("Close Events Table Frame") );
    }
//    this.toBack();
//	if (control != null) { control.setFront(); }
  }

  /* ===================================================================== *
   * ------------------------ OBSERVER ACTIONS --------------------------- *
   * ===================================================================== */

  /** Method update()
   * <p> This method will handle the Observable Actions
   * @param obj = The Observable making the call
   * @param arg = The returned object
   */

  public void update(Observable obj, Object arg)
  {
    String             sArg    = new String( (String) arg);
    int                iSelect = -1;
    String             sFile   = "";
    seisReadMSeed      pIn     = null;

    iqstratFilesStruct st      = null;
    String             sPath   = "";
    psiCSVFile         pCSV    = null;

    if (sArg.equals("Event Selected"))
    {
	  iSelect = pCatalog.getSelectedRow();
	  iEvent  = iSelect;
      sFile   = getMiniSeedFile( stCatalog.tsUTC[iSelect],
                                 stSensor.sid[iSensor],
                                 stSensor.sName[iSensor] );

      if (sFile.length() > 0)
      {
        pIn     = new seisReadMSeed();
        pIn.OpenURL(sFile);                                        // Open File
        stWave = seismic.seisUtility.copyData( pIn.getData() );    // Get Data
        pIn.Close();                                               // Close File

        st = getPressureFile( stCatalog.ts[iSelect] );

        if (st != null)
        {
		  sPath    = new String(st.sURL);
		  sFile    = new String(st.sFilename);
		  iUTC_hrs = st.iUTC_hrs;

          pCSV     = new psiCSVFile();

          pCSV.Open(util.utilFileIO.URL_FILE, sPath, sFile);       // Open File
          pCSV.get2Rows();                                         // Count Rows
          pCSV.Close();                                            // Close File

          pCSV.Open(util.utilFileIO.URL_FILE, sPath, sFile);       // Open File
          pCSV.getData(1);                                         // Build Data Structure
          stSeis = seismic.seisUtility.copyData( pCSV.getData() ); // Get Data
          pCSV.Close();                                            // Close File
	    }

        if ((stSeis != null) && (stWell != null))
        {
		  stSeis.location = new String(stWell.sName);
	    }

        plot();
        this.toBack();
	  }
	}

//	if (control != null) { control.setFront(); }
  }
}

/** CLASS psiFrameObservable()
  *  <p> This Class will handle Observable Actions when action occurs on a
  *      frame or panel
  */

class psiFrameObservable extends Observable
{
  public void notifyObservers(Object b)
  {
    setChanged();
    super.notifyObservers(b);
  }
}

/*
 *  @version 1.1 5/27/2016
 *  @author  John Victorine
 */
