/*
 * @psiMath.java Version 1.1 05/12/2016
 *
 * Copyright (c) 2016 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package psi.math;

import java.sql.Timestamp;

import math.Complex;

import seismic.seisStruct;

/** Class mathSS
 *  <p> This Class provides methods for filtering the seismic data.
 *
 *  @version 1.0 05/12/2016
 *  @author  John R. Victorine
 */

public class psiMath
{
  public static final int _NONE   = -1;
  public static final int _PULSE  = 0;
  public static final int _SQR    = 1;
  public static final int _SQR_2  = 2;
  public static final int _WAVE   = 3;
  public static final int _KALMAN = 4;

  /** Method envelope()
   *  <p> This method will compute the envelope of the seismic wave
   *  @param  stIn  = seismic data structure
   *  @return stOut = new seismic data structure
   */

  public static seisStruct envelope(seisStruct stIn)
  {
	int i=0;
    seisStruct stOut   = null;
    int        iCount  = 0;

    int        iRows   = 0;
    Timestamp  ts[]    = null;
    double     data[]  = null;

    int        iRowse  = 0;
    Timestamp  tse[]   = null;
    double     datae[] = null;

    double     del1    = 0.0;
    double     del2    = 0.0;
    double     dTemp   = 0.0;
    double     dAvg    = 0.0;

    if (stIn != null)
    {
      stOut          = new seisStruct();

      stOut.network  = new String(stIn.network);  // Network Code
      stOut.station  = new String(stIn.station);  // Station Identifier
      stOut.location = new String(stIn.location); // Location Identifier
      stOut.channel  = new String(stIn.channel);  // Channel Identifier

      stOut.tsStart  = new Timestamp(stIn.tsStart.getTime());
      stOut.tsEnd    = new Timestamp(stIn.tsEnd.getTime());

      stOut.dMaximum = stIn.dMaximum; // Maximum Data Value
      stOut.dMinimum = stIn.dMinimum; // Minimum Data Value

      // Seismic Records

      iRows = stIn.iRows;           // Total Number of rows
      ts    = new Timestamp[iRows]; // Time Stamp of data record
      data  = new double[iRows];    // Data record

      for (i=0; i<iRows; i++)
      {
        ts[i]   = new Timestamp(stIn.ts[i].getTime());
        dAvg    = ( stOut.dMaximum + stOut.dMinimum  ) / 2.0;
		data[i] = Math.abs( stIn.data[i] - dAvg );
//		data[i] = stIn.data[i];
      }

      tse   = new Timestamp[iRows]; // Time Stamp of data record
      datae = new double[iRows];    // Data record

      iCount = 0;
      for (i=1; i<iRows-1; i++)
      {
		del2 = data[i+1] - data[i];
		del1 = data[i]   - data[i-1];

		if ((del1 >= 0.0) && (del2 < 0.0))
		{
          tse[iCount]   = new Timestamp(ts[i].getTime());
		  datae[iCount] = data[i];
		  dTemp         = data[i];
		  iCount++;
		}
//		else
//		{
//          tse[iCount]   = new Timestamp(ts[i].getTime());
//		  datae[iCount] = dTemp;
//		  iCount++;
//		}
	  }

      stOut.iRows    = iCount;                 // Total Number of rows
      stOut.ts       = new Timestamp[iCount]; // Time Stamp of data record
      stOut.data     = new double[iCount];    // Data record

      for (i=0; i<iCount; i++)
      {
        stOut.ts[i]   = new Timestamp(tse[i].getTime());
		stOut.data[i] = datae[i];
	  }
    }

    for (int j=0; j<stOut.iRows; j++)
    {
      if (j==0)
      {
  	    stOut.dMinimum = stOut.data[j];
  	    stOut.dMaximum = stOut.data[j];
	  }

  	  if (stOut.data[j] < stOut.dMinimum)
  	    stOut.dMinimum = stOut.data[j];

      if (stOut.data[j] > stOut.dMaximum)
  	    stOut.dMaximum = stOut.data[j];
	}

	stOut.dMinimum = (double) ((int) stOut.dMinimum - 1);
  	stOut.dMaximum = (double) ((int) stOut.dMaximum + 1);

    return (stOut);
  }

  /** Method movingAverage()
   *  <p> This method will compute the moving average across the dataset
   *  @param  stIn  = seismic data structure
   *  @param  iSum  = the total number of points to sum
   *  @param  data  = array of data points to convolve with seismic data
   *  @return stOut = new seismic data structure
   */

  public static seisStruct movingAverage(seisStruct stIn, int iSum, double data[])
  {
	int        jn     = 0;
    int        kn     = 0;
    seisStruct stOut  = null;
    int        iCount = 0;
    double     dSum   = 0.0;

    if (stIn != null)
    {
      stOut          = new seisStruct();

      stOut.network  = new String(stIn.network);  // Network Code
      stOut.station  = new String(stIn.station);  // Station Identifier
      stOut.location = new String(stIn.location); // Location Identifier
      stOut.channel  = new String(stIn.channel);  // Channel Identifier

      stOut.tsStart  = new Timestamp(stIn.tsStart.getTime());
      stOut.tsEnd    = new Timestamp(stIn.tsEnd.getTime());

      stOut.dMaximum = stIn.dMaximum; // Maximum Data Value
      stOut.dMinimum = stIn.dMinimum; // Minimum Data Value

      // Seismic Records

      stOut.iRows    = stIn.iRows;                 // Total Number of rows
      stOut.ts       = new Timestamp[stOut.iRows]; // Time Stamp of data record
      stOut.data     = new double[stOut.iRows];    // Data record

      for (int i=0; i<stIn.iRows; i++)
      {
        stOut.ts[i]  = new Timestamp(stIn.ts[i].getTime());
      }

      for (int i=0; i<stIn.iRows; i++)
      {
        iCount = 0;
        dSum   = 0.0;
        for (int j=-iSum/2; j<1+iSum/2; j++)
        {
          kn = i+j;
          jn = j+iSum/2;
          if ((kn > 0) && (kn < stIn.iRows))
          {
			if ((jn < iSum) && (jn > 0))
			{
              dSum = dSum + stIn.data[kn] * data[jn];
              iCount++;
		    }
          }
        }
        stOut.data[i] = dSum / iCount;
      }
    }

    for (int j=0; j<stOut.iRows; j++)
    {
      if (j==0)
      {
  	    stOut.dMinimum = stOut.data[j];
  	    stOut.dMaximum = stOut.data[j];
	  }

  	  if (stOut.data[j] < stOut.dMinimum)
  	    stOut.dMinimum = stOut.data[j];

      if (stOut.data[j] > stOut.dMaximum)
  	    stOut.dMaximum = stOut.data[j];
	}

	stOut.dMinimum = (double) ((int) stOut.dMinimum);
  	stOut.dMaximum = (double) ((int) stOut.dMaximum);

    return (stOut);
  }

  /** Method loadData()
   *  <p> This method will build the wave form from the seismic data wave
   *      to use in filtering the data
   *  @param  iStart = Starting Point of wave
   *  @param  iEnd   = Ending point of wave
   *  @param  dMin   = Minimum Magnitude of Normalization
   *  @param  dMax   = Maximum Magnitude of Normalization
   *  @param  st     = the siesmic data structure
   *  @return data   = the data set constructed from the seismic data wave
   */

  public static double[] loadData( int    iStart, int    iEnd,
                                   double dMin,   double dMax,
                                   seisStruct st)
  {
	double data[]   = null;
	double dtmp[]   = null;
	int    jData    = 0;
	int    iCount   = 0;
	double dMinimum     = 0.0;
	double dMaximum     = 0.0;

	if (st != null)
	{
	  jData = iEnd - iStart + 1;
      dtmp  = new double[jData];

      for (int i=0; i<st.iRows; i++)
      {
		if (iCount < jData)
		{
		  dtmp[iCount] = st.data[i];

		  if (dtmp[iCount] < dMinimum) { dMinimum = dtmp[iCount]; }
		  if (dtmp[iCount] > dMaximum) { dMaximum = dtmp[iCount]; }

		  iCount++;
		}
	  }

      // Normalize data to the Min and Max passed in

      data = new double[jData];

      for (int j=0; j<iCount; j++)
      {
		data[j] = dMin + (dMax-dMin)*((dtmp[j]-dMinimum)/(dMaximum-dMinimum));
	  }
	}

	return (data);
  }

  /** Method movingAverage()
   *  <p> This method will compute the moving average across the dataset
   *  @param stIn  = seismic data structure
   *  @param iWave = the type of averaging to perform
   *                 PULSE = Simple moving average across the number of points
   *                 SQR   = Square Wave first half of points negative 1,
   *                         second half of points positive 1.
   *			     SQR_2 = Two sets of Square Wave
   *  @param iSum  = the total points
   *  @param stOut = new seismic data structure
   */

  public static seisStruct movingAverage(seisStruct stIn, int iWave, int iSum)
  {
    int        kn     = 0;
    seisStruct stOut  = null;
    int        iCount = 0;
    double     dSum   = 0.0;
    double     dMean  = 0.0;

    if (stIn != null)
    {
      stOut          = new seisStruct();

      stOut.network  = new String(stIn.network);  // Network Code
      stOut.station  = new String(stIn.station);  // Station Identifier
      stOut.location = new String(stIn.location); // Location Identifier
      stOut.channel  = new String(stIn.channel);  // Channel Identifier

      stOut.tsStart  = new Timestamp(stIn.tsStart.getTime());
      stOut.tsEnd    = new Timestamp(stIn.tsEnd.getTime());

      stOut.dMaximum = stIn.dMaximum; // Maximum Data Value
      stOut.dMinimum = stIn.dMinimum; // Minimum Data Value

      // Seismic Records

      stOut.iRows    = stIn.iRows;                 // Total Number of rows
      stOut.ts       = new Timestamp[stOut.iRows]; // Time Stamp of data record
      stOut.data     = new double[stOut.iRows];    // Data record

      for (int i=0; i<stIn.iRows; i++)
      {
        stOut.ts[i]  = new Timestamp(stIn.ts[i].getTime());
      }

      for (int i=0; i<stIn.iRows; i++)
      {
        iCount = 0;
        dSum   = 0.0;
        for (int j=-iSum/2; j<1+iSum/2; j++)
        {
          kn = i+j;
          if ((kn > 0) && (kn < stIn.iRows))
          {
            dSum = dSum + stIn.data[kn];
            iCount++;
		  }
		}
        dMean = dSum / iCount;

        iCount = 0;
        dSum   = 0.0;
        for (int j=-iSum/2; j<1+iSum/2; j++)
        {
          kn = i+j;
          if ((kn > 0) && (kn < stIn.iRows))
          {
			switch (iWave)
			{
              case _PULSE:
                dSum = dSum + stIn.data[kn];
                break;
              case _SQR:
  			    if (j>0)
                  dSum = dSum + stIn.data[kn] + dMean;
                else
                  dSum = dSum - stIn.data[kn] + dMean;
                break;
              case _SQR_2:
			    if (j>0)
			    {
			      if (j>iSum/4)
			      {
                    dSum = dSum + stIn.data[kn] + dMean;
			      }
                  else
                  {
                    dSum = dSum - stIn.data[kn] + dMean;
			      }
			    }
                else
                {
			      if (j>-iSum/4)
			      {
                    dSum = dSum + stIn.data[kn] + dMean;
			      }
                  else
                  {
                    dSum = dSum - stIn.data[kn] + dMean;
			      }
			    }
                break;
		    }

            iCount++;
          }
        }
        stOut.data[i] = dSum / iCount;
      }
    }

    for (int j=0; j<stOut.iRows; j++)
    {
      if (j==0)
      {
  	    stOut.dMinimum = stOut.data[j];
  	    stOut.dMaximum = stOut.data[j];
	  }

  	  if (stOut.data[j] < stOut.dMinimum)
  	    stOut.dMinimum = stOut.data[j];

      if (stOut.data[j] > stOut.dMaximum)
  	    stOut.dMaximum = stOut.data[j];
	}

	stOut.dMinimum = (double) ((int) stOut.dMinimum);
  	stOut.dMaximum = (double) ((int) stOut.dMaximum);

    return (stOut);
  }

  /** Method convolve()
   *  <p> This method will compute the frequency spectrum of data
   *  @param stIn  = seismic data structure
   *  @param start = starting date/time
   *  @param end   = ending date/time
   *  @param stOut = seismic data structure
   */

  public static seisStruct convolve( seisStruct stIn,
                                     Timestamp start, Timestamp end )
  {
	int i,j;
    seisStruct stOut  = null;
    int        kn     = 0;
    double     data[] = null;
    int        iTotal = 0;
    int        iSum   = 0;
    int        Lo     = 0;
    int        Lm     = 0;
    double     dSum   = 0.0;
    double     dRef   = 0.0;
    int        iCount = 0;

    long       ls     = (long) 0;
    long       le     = (long) 0;
    long       ldiff  = (long) 0;

    double     delT   = 0.0;

    if (stIn != null)
    {
      stOut          = new seisStruct();

      stOut.network  = new String(stIn.network);  // Network Code
      stOut.station  = new String(stIn.station);  // Station Identifier
      stOut.location = new String(stIn.location); // Location Identifier
      stOut.channel  = new String(stIn.channel);  // Channel Identifier

      stOut.tsStart  = new Timestamp(stIn.tsStart.getTime());
      stOut.tsEnd    = new Timestamp(stIn.tsEnd.getTime());

      stOut.dMaximum = stIn.dMaximum; // Maximum Data Value
      stOut.dMinimum = stIn.dMinimum; // Minimum Data Value

      // Seismic Records

      stOut.iRows    = stIn.iRows;                 // Total Number of rows
      stOut.ts       = new Timestamp[stOut.iRows]; // Time Stamp of data record
      stOut.data     = new double[stOut.iRows];    // Data record

      for (i=0; i<stIn.iRows; i++)
      {
        stOut.ts[i]   = new Timestamp(stIn.ts[i].getTime());
        stOut.data[i] = 0.0;
      }

	  ls    = stIn.ts[0].getTime();
	  le    = stIn.ts[1].getTime();
	  ldiff = le - ls;
	  delT  = (double) (ldiff) / 1000.0; // convert time to seconds.

      for (i=0; i<stIn.iRows; i++)
      {
  	    if (stIn.ts[i].compareTo(start) <= 0)
		  Lo = i;

		if (stIn.ts[i].compareTo(end) <= 0)
		  Lm = i;
	  }

      iTotal = Lm-Lo;

      data   = new double[iTotal];
      iCount = 0;
      for (i=0; i<stIn.iRows; i++)
      {
		if ((i >= Lo) && (i<Lm))
		{
		  if (iCount < iTotal)
		  {
		    data[iCount] = stIn.data[i];
		    if (dRef < data[iCount])
		      dRef = data[iCount];
//		    dRef         = dRef + data[iCount]*data[iCount];
		    iCount++;
		  }
	    }
	  }

//      dRef = Math.sqrt(dRef)/iCount;

      for (i=0; i<stIn.iRows; i++)
      {
        dSum   = 0.0;

        for (j=0; j<iTotal; j++)
        {
          kn = i+j-iTotal/2;
          if ((kn >= 0) && (kn < stIn.iRows))
          {
            dSum = dSum + stIn.data[kn] * data[j] / dRef; //* delT / iTotal;
		  }
		}

		stOut.data[i] = dSum;
	  }
	}

    for (j=0; j<stOut.iRows; j++)
    {
      if (j==0)
      {
  	    stOut.dMinimum = stOut.data[j];
  	    stOut.dMaximum = stOut.data[j];
	  }

  	  if (stOut.data[j] < stOut.dMinimum)
  	    stOut.dMinimum = stOut.data[j];

      if (stOut.data[j] > stOut.dMaximum)
  	    stOut.dMaximum = stOut.data[j];
	}

	if (stOut.dMinimum == stOut.dMaximum)
	  stOut.dMaximum = stOut.dMinimum + 1.0;

	stOut.dMinimum = (double) ((int) stOut.dMinimum);
  	stOut.dMaximum = (double) ((int) stOut.dMaximum);

    return (stOut);
  }

  /** Method mean()
   *  <p> This method computes the mean value of the data set
   *  @param  stIN  = the Seismic Data Structure or the Raw Data
   *  @return dAvg  = the mean value of the data set
   */

  public static double mean(seisStruct stIn)
  {
	double dAvg = 0.0;
	double dSum = 0.0;
	int    icnt = 0;

	if (stIn != null)
	{
      for (int i=0; i<stIn.iRows; i++)
      {
        dSum = dSum + stIn.data[i];
        icnt++;
	  }

	  dAvg = dSum / icnt;
	  dAvg = (double) ((int) (dAvg * 100.0)/100.0);
	}

	return (dAvg);
  }

  /** Method kalman()
   *  <p> This method is a simple one-dimensional Kalman Filter
   *  @param  stIN  = the Seismic Data Structure or the Raw Data
   *  @param  qo    = process noise covarianvce (0.00001)
   *  @param  ro    = measurement noise covariance (0.01)
   *  @param  po    = estimation error covariance (1)
   *  @param  xo    = value initial quess at level
   *  @return stOut = the Seismic Data Structure w/o Lunar Cycle and Mean value
   */

  public static seisStruct kalman(seisStruct stIn, double qo, double ro, double po, double xo)
  {
	seisStruct stOut  = null;
	double     q      = 0.0; // process noise covariance
	double     r      = 0.0; // measurement noise covariance
	double     p      = 0.0; // estimation error covariance
	double     x      = 0.0; // value initial quess at level
	double     K      = 0.0; // kalman gain

	if (stIn != null)
	{
      q = qo;
      r = ro;
      p = po;
      x = xo;

      stOut          = new seisStruct();

      stOut.network  = new String(stIn.network);  // Network Code
      stOut.station  = new String(stIn.station);  // Station Identifier
      stOut.location = new String(stIn.location); // Location Identifier
      stOut.channel  = new String(stIn.channel);  // Channel Identifier

      stOut.tsStart  = new Timestamp(stIn.tsStart.getTime());
      stOut.tsEnd    = new Timestamp(stIn.tsEnd.getTime());

      stOut.dMaximum = stIn.dMaximum; // Maximum Data Value
      stOut.dMinimum = stIn.dMinimum; // Minimum Data Value

      // Seismic Records

      stOut.iRows    = stIn.iRows;                 // Total Number of rows
      stOut.ts       = new Timestamp[stOut.iRows]; // Time Stamp of data record
      stOut.data     = new double[stOut.iRows];    // Data record

      for (int i=0; i<stIn.iRows; i++)
      {
        stOut.ts[i]  = new Timestamp(stIn.ts[i].getTime());

        p = p + q;

        K = p / ( p + r );
        x = x + K * ( stIn.data[i] - x );
        p = (1 - K) * p;

		stOut.data[i] = x;
	  }
	}

	return (stOut);
  }

  /** Method difference()
   *  <p> This method will compute the difference between two frequency
   *      spectrums.  The data arrays must be computed at the same frequency
   *      range and increments.  This routine is just a simple subtraction
   *      of 2 matracies.
   */

  public static double[][] difference(double d1[][], double d2[][])
  {
	int i,j;
    double data[][] = null;
    double dReal    = 0.0;
    double dImag    = 0.0;

    if ((d1 != null) && (d2 != null))
    {
	  if (d1.length == d2.length)
	  {
		data = new double[d1.length][3];
		for (i=0; i<d1.length; i++)
		{
		  for (j=0; j<3; j++)
		  {
			data[i][j] = 0.0;
			if (j == 0)
			{
			  data[i][j] = d1[i][j];
			}
		  }
		}

		for (i=0; i<d1.length; i++)
		{
		  dReal = d1[i][1]*Math.cos(d1[i][2]) - d2[i][1]*Math.cos(d2[i][2]);
		  dImag = d1[i][1]*Math.sin(d1[i][2]) - d2[i][1]*Math.sin(d2[i][2]);

		  data[i][1] = Math.hypot(dReal, dImag);
		  data[i][2] = Math.atan2(dImag, dReal);
		}
	  }
	}

	return (data);
  }

  /** Method frequencySpace()
   *  <p> This method will compute the frequency spectrum of data
   *  @param stIn  = seismic data structure
   *  @param start = starting date/time
   *  @param end   = ending date/time
   *  @param fi    = Initial (Starting) Frequency (Hz)
   *  @param fm    = Maximum (Ending) Frequency (Hz)
   *  @param Npts  = Total number of points to "integrate" the signal over
   *  @param data  = array of frequecy, magnitude & phase
   */

  public static double[][] frequencySpace( seisStruct stIn,
                                           Timestamp start, Timestamp end,
                                           double fi, double fm, int Npts )
  {
	int        i,j,L,M;
    double     data[][] = null;

	int        Lo     = 0;
	int        Lu     = 0;
	int        kn     = 0;

    long       ls     = (long) 0;
    long       le     = (long) 0;
    long       ldiff  = (long) 0;

    double     delta  = 0.0;
    double     delT   = 0.0;

    double     fil    = Math.log(fi)/Math.log(10);
    double     fml    = Math.log(fm)/Math.log(10);
    double     fdiffl = 0.0;

    double     fvl    = 0.0;
    double     fv     = 0.0;
    double     wm     = 0.0;

    double     dReal  = 0.0;
    double     dImag  = 0.0;
    double     Lpts   = 0;

    if ((stIn != null) && (start != null) && (end != null) && (Npts > 0))
    {
      data  = new double[Npts][3];  // Columns = {Frequency, Magnitude, Phase }
	  for (M=0; M<Npts; M++)
	  {
	    for (j=0; j<3; j++)
	    {
		  data[M][j] = 0.0;
		}
	  }

      fdiffl = (fml - fil) / Npts; // Compute the frequency increment

      if (stIn.iRows > 0)
      {
		ls    = stIn.ts[0].getTime();
		le    = stIn.ts[1].getTime();
		ldiff = le - ls;
		delT  = (double) (ldiff) / 1000.0; // convert time to seconds.

		Lo    = 0;
		Lu    = stIn.iRows;

	    for (i=0; i<stIn.iRows; i++)
		{
		  if (stIn.ts[i].compareTo(start) == 0)
		  {
			Lo = i;
		  }
		  else if (stIn.ts[i].compareTo(start) < 0)
		  {
			Lo = i;
		  }

		  if (stIn.ts[i].compareTo(end) == 0)
		  {
			Lu = i;
		  }
		  else if (stIn.ts[i].compareTo(end) < 0)
		  {
			Lu = i;
		  }
		}

        Lu++;
        if (Lu > stIn.iRows)
          Lu = stIn.iRows;
        Lpts = Lu - Lo;

		if ((start != null) && (end != null))
		{
		  ls    = start.getTime();
		  le    = end.getTime();
		  ldiff = le-ls;

		  delta = (double) (ldiff) / 1000.0;
		}

		ldiff = le-ls;

		for (M=0; M<Npts; M++)
		{
	      fvl   = fil + M * fdiffl;
	      fv    = Math.pow(10.0, fvl);
//System.out.println(fv);
          dReal = 0.0;
          dImag = 0.0;

		  for (i=0; i<stIn.iRows; i++)
		  {
			if ((i >= Lo) && (i < Lu))
		    {
		      kn    = (i-Lo) - (int) (Lpts/2);
		      wm    = -2.0 * Math.PI * fv * kn * delT;
		      dReal = dReal + stIn.data[i] * Math.cos(wm) * delT;
		      dImag = dImag + stIn.data[i] * Math.sin(wm) * delT;
			}
		  }

		  data[M][0] = fv;
		  data[M][1] = Math.hypot(dReal, dImag);
		  data[M][2] = Math.atan2(dImag, dReal);
//System.out.println(M+" "+data[M][0]+" "+data[M][1]+" "+data[M][2]);
		}
	  }
	}

    return (data);
  }

  /** Method timeSpace()
   *  <p> This method will compute the moving average across the dataset
   *  @param stIn  = seismic data structure
   *  @param data  = Frequence Space Data {Frequency (Hz), Magnitude, Phase}
   *  @param stOut = new seismic data structure
   */

  public static seisStruct timeSpace(seisStruct stIn, double data[][])
  {
	int i=0;
    seisStruct stOut  = null;
    double     fv     = 0.0;
    double     fdiff  = 0.0;
    double     cSum   = 0.0;
    double     wm     = 0.0;

    double     fil    = 0.0;
    double     fml    = 0.0;
    double     fdiffl = 0.0;

    long       ls     = (long) 0;
    long       le     = (long) 0;
    long       ldiff  = (long) 0;
    double     delT   = 0.0;

    int        kn     = 0;
    int        Npts   = 0;

    if (data != null)
    {
	  Npts = data.length;
	}

    if ((stIn != null) && (Npts > 0))
    {
      stOut = new seisStruct();

      stOut.network  = new String(stIn.network);  // Network Code
      stOut.station  = new String(stIn.station);  // Station Identifier
      stOut.location = new String(stIn.location); // Location Identifier
      stOut.channel  = new String(stIn.channel);  // Channel Identifier

      stOut.tsStart  = new Timestamp(stIn.tsStart.getTime());
      stOut.tsEnd    = new Timestamp(stIn.tsEnd.getTime());

      stOut.dMaximum = stIn.dMaximum; // Maximum Data Value
      stOut.dMinimum = stIn.dMinimum; // Minimum Data Value

      // Seismic Records

      stOut.iRows    = stIn.iRows;                 // Total Number of rows
      stOut.ts       = new Timestamp[stOut.iRows]; // Time Stamp of data record
      stOut.data     = new double[stOut.iRows];    // Data record

      for (i=0; i<stIn.iRows; i++)
      {
        stOut.ts[i]   = new Timestamp(stIn.ts[i].getTime());
        stOut.data[i] = 0.0;
      }

      ls     = stIn.ts[0].getTime();
      le     = stIn.ts[1].getTime();
      ldiff  = le-ls;

      delT   = (double) (ldiff / 1000.0); // convert time to seconds.

      fil    = Math.log(data[0][0])/Math.log(10);
      fml    = Math.log(data[Npts-1][0])/Math.log(10);
      fdiffl = (fml - fil) / Npts;

      fdiff  = Math.pow(10.0, fdiffl);

      if (stIn.iRows > 0)
      {
        for (i=0; i<stIn.iRows; i++)
        {
		  cSum = 0;

		  for (int N=0; N<Npts; N++)
		  {
			fv   = data[N][0];
			if (N==0)
			{
			  fdiff = data[1][0] - data[0][0];
			}
			else
			{
			  fdiff = data[N][0] - data[N-1][0];
			}

  	        kn   = i - (int) (stIn.iRows/2);
			wm   = 2.0 * Math.PI * fv * i * delT;

			cSum = cSum + data[N][1]*Math.cos(data[N][2] + wm) * fdiff;
//if (i==10)
//{
//System.out.println(cSum+" "+ data[N][1]+" "+data[N][2]+" "+wm+" "+fdiff);
//}
		  }

		  stOut.data[i] = cSum;
		}
	  }
	}

    return (stOut);
  }

  /** Method fourier()
   *  <p> This method will compute the moving average across the dataset
   *  @param stIn  = seismic data structure
   *  @param fi    = Initial (Starting) Frequency (Hz)
   *  @param fm    = Maximum (Ending) Frequency (Hz)
   *  @param Npts  = Total number of points to "integrate" the signal over
   *  @param stOut = new seismic data structure
   */

  public static seisStruct fourier(seisStruct stIn, double fi, double fm, int Npts)
  {
    seisStruct stOut  = null;

	int        No     = 0;
	int        Nu     = 0;
	int        kn     = 0;

    double     fil    = 0.0;
    double     fml    = 0.0;
    double     fdiffl = 0.0;

    double     fv     = 0.0;

    long       ls     = (long) 0;
    long       le     = (long) 0;
    long       ldiff  = (long) 0;

    double     delT   = 0;
    double     delfv  = 0.0;

    double     cSum   = 0.0;
    double     wm     = 0.0;

    if (stIn != null)
    {
      stOut = new seisStruct();

      stOut.network  = new String(stIn.network);  // Network Code
      stOut.station  = new String(stIn.station);  // Station Identifier
      stOut.location = new String(stIn.location); // Location Identifier
      stOut.channel  = new String(stIn.channel);  // Channel Identifier

      stOut.tsStart  = new Timestamp(stIn.tsStart.getTime());
      stOut.tsEnd    = new Timestamp(stIn.tsEnd.getTime());

      stOut.dMaximum = stIn.dMaximum; // Maximum Data Value
      stOut.dMinimum = stIn.dMinimum; // Minimum Data Value

      // Seismic Records

      stOut.iRows    = stIn.iRows;                 // Total Number of rows
      stOut.ts       = new Timestamp[stOut.iRows]; // Time Stamp of data record
      stOut.data     = new double[stOut.iRows];    // Data record

      for (int i=0; i<stIn.iRows; i++)
      {
        stOut.ts[i]  = new Timestamp(stIn.ts[i].getTime());
      }

      if (stIn.iRows > 0)
      {
		No = -1*Npts/2;
		Nu = Npts/2;

        fil    = Math.log(fi)/Math.log(10);
        fml    = Math.log(fm)/Math.log(10);
        fdiffl = (fml - fil) / Npts;

		ls     = stIn.ts[0].getTime();
		le     = stIn.ts[1].getTime();
		ldiff  = le-ls;

		delT   = (double) (ldiff / 1000.0); // convert time to seconds.
		delfv  = Math.pow(10.0, fdiffl);

        for (int i=0; i<stIn.iRows; i++)
        {
		  cSum = 0.0;

  		  for (int M=0; M<Npts; M++)
		  {
			fv = fi + M * delfv;

            for (int L=No; L<Nu; L++)
            {
              kn = i+L;
              if ((kn >= 0) && (kn < stIn.iRows))
              {
			    wm = 2.0 * Math.PI * fv * L * delT;
			    cSum = cSum + stIn.data[kn] * Math.cos(wm) * delT * delfv;
			  }
			}
		  }

		  stOut.data[i] = cSum;
		}
	  }
	}

	return (stOut);

  }
}


/*
 *  @version 1.1 05/12/2016
 *  @author  John Victorine
 */
