/*
 * @psiUtility.java Version 1.1 05/18/2016
 *
 * Copyright (c) 2016 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package psi;

import java.sql.Timestamp;
import java.sql.Date;
import java.text.SimpleDateFormat;
import java.text.ParseException;

import seismic.seisStruct;  // Seismic Data Structure

/** Class psiUtility
 *  <p> This is the seismic data structure utility class for
 *      IRIS (Incorporated Research Institutions for Seismology) Data.
 *
 *  @version 1.1 05/18/2016
 *  @author  John R. Victorine
 */

public class psiUtility
{
  public static final int _PULSE = psi.math.psiMath._PULSE;

  /** Method removeLunarCycle()
   *  <p> This method will remove the "Lunar" Cycle from the pressure, but
   *      it really removes the running "Mean" value of the curve to leave
   *      noise and any other signals imbedded in the signal.
   *  @param  stIN  = the Seismic Data Structure or the Raw Data
   *  @return stOut = the Seismic Data Structure w/o Lunar Cycle and Mean value
   */

  public static seisStruct removeLunarCycle(seisStruct stIn)
  {
	seisStruct st10  = null;
	seisStruct st100 = null;
	seisStruct stOut = null;

	if (stIn != null)
	{
	  st10  = psi.math.psiMath.movingAverage( stIn, _PULSE, 10 );
	  st100 = psi.math.psiMath.movingAverage( stIn, _PULSE, 100 );

      stOut          = new seisStruct();

      stOut.network  = new String(stIn.network);  // Network Code
      stOut.station  = new String(stIn.station);  // Station Identifier
      stOut.location = new String(stIn.location); // Location Identifier
      stOut.channel  = new String(stIn.channel);  // Channel Identifier

      stOut.tsStart  = new Timestamp(stIn.tsStart.getTime());
      stOut.tsEnd    = new Timestamp(stIn.tsEnd.getTime());

      stOut.dMaximum = stIn.dMaximum; // Maximum Data Value
      stOut.dMinimum = stIn.dMinimum; // Minimum Data Value

      // Seismic Records

      stOut.iRows    = stIn.iRows;                 // Total Number of rows
      stOut.ts       = new Timestamp[stOut.iRows]; // Time Stamp of data record
      stOut.data     = new double[stOut.iRows];    // Data record

      for (int i=0; i<stIn.iRows; i++)
      {
        stOut.ts[i]  = new Timestamp(stIn.ts[i].getTime());
      }

      for (int i=0; i<stIn.iRows; i++)
      {
		stOut.data[i] = st10.data[i] - st100.data[i];
	  }
	}

	return (stOut);
  }

  /** Method removeLunarCycle(seisStruct stIn, double dMag, String sDate)
   *  <p> This method will remove only the lunar pressure influence
   *      this a simple method assuming a sin wave centered at a specific
   *      date.
   *
   *     Pl = Pm * sine ( 2 * PI * (t(hrs) - to(hrs)) / 12)
   *        Pm is dMag
   *        to is timestamp of sDate in long divided by 3600000 to convert seconds to hrs.
   *
   *  @param  stIn  = the Seismic Data Structure or the Raw Data
   *  @param  dMag  = the lunar pressure influence
   *  @param  sDate = the "0" date (Start of cycle) format: yyyy-MM-dd HH:mm:ss.SSSSSS
   *  @return stOut = the Seismic Data Structure w/o Lunar Cycle
   */

  public static seisStruct removeLunarCycle(seisStruct stIn, double dMag, String sDate)
  {
	seisStruct       stOut  = null;
    SimpleDateFormat dfm    = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSSSSS");
    long             lto    = (long) 0;
    long             ltime  = (long) 0;
    long             lhr    = (long) 3600000;
    double           delt   = 0.0;
    double           dlunar = 0.0;
    long             ltemp  = (long) 0;
    double           dtemp  = 0.0;

	if ((stIn != null) && (sDate.length() > 0))
	{
      try
      {
        lto = dfm.parse(sDate).getTime();
      }
      catch (ParseException e)
      {
        e.printStackTrace();
      }

      stOut          = new seisStruct();

      stOut.network  = new String(stIn.network);  // Network Code
      stOut.station  = new String(stIn.station);  // Station Identifier
      stOut.location = new String(stIn.location); // Location Identifier
      stOut.channel  = new String(stIn.channel);  // Channel Identifier

      stOut.tsStart  = new Timestamp(stIn.tsStart.getTime());
      stOut.tsEnd    = new Timestamp(stIn.tsEnd.getTime());

      stOut.dMaximum = stIn.dMaximum; // Maximum Data Value
      stOut.dMinimum = stIn.dMinimum; // Minimum Data Value

      // Seismic Records

      stOut.iRows    = stIn.iRows;                 // Total Number of rows
      stOut.ts       = new Timestamp[stOut.iRows]; // Time Stamp of data record
      stOut.data     = new double[stOut.iRows];    // Data record

      for (int i=0; i<stIn.iRows; i++)
      {
        stOut.ts[i]   = new Timestamp(stIn.ts[i].getTime());

        ltime         = stOut.ts[i].getTime();

dtemp = (double) (ltime - lto) / (double) lhr;
//ltemp = (ltime - lto) / lhr;
//System.out.println(stOut.ts[i].toString()+" "+dtemp);

//        delt          = ( Math.PI / 6.0 ) * (double) ( (ltime - lto) / lhr );
        delt          = Math.PI  * dtemp / 6.0;
        dlunar        = dMag * Math.sin(delt);

		stOut.data[i] = stIn.data[i] - dlunar;
	  }
	}

	return (stOut);
  }

  /** Method kalman()
   *  <p> This method is a simple one-dimensional Kalman Filter
   *  @param  stIN  = the Seismic Data Structure or the Raw Data
   *  @param  qo    = process noise covarianvce (0.00001)
   *  @param  ro    = measurement noise covariance (0.01)
   *  @param  po    = estimation error covariance (1)
   *  @param  xo    = value initial quess at level
   *  @return stOut = the Seismic Data Structure w/o Lunar Cycle and Mean value
   */

  public static seisStruct kalman(seisStruct stIn, double qo, double ro, double po, double xo)
  {
	seisStruct stOut  = null;
	double     q      = 0.0; // process noise covariance
	double     r      = 0.0; // measurement noise covariance
	double     p      = 0.0; // estimation error covariance
	double     x      = 0.0; // value initial quess at level
	double     K      = 0.0; // kalman gain

	if (stIn != null)
	{
      q = qo;
      r = ro;
      p = po;
      x = xo;

      stOut          = new seisStruct();

      stOut.network  = new String(stIn.network);  // Network Code
      stOut.station  = new String(stIn.station);  // Station Identifier
      stOut.location = new String(stIn.location); // Location Identifier
      stOut.channel  = new String(stIn.channel);  // Channel Identifier

      stOut.tsStart  = new Timestamp(stIn.tsStart.getTime());
      stOut.tsEnd    = new Timestamp(stIn.tsEnd.getTime());

      stOut.dMaximum = stIn.dMaximum; // Maximum Data Value
      stOut.dMinimum = stIn.dMinimum; // Minimum Data Value

      // Seismic Records

      stOut.iRows    = stIn.iRows;                 // Total Number of rows
      stOut.ts       = new Timestamp[stOut.iRows]; // Time Stamp of data record
      stOut.data     = new double[stOut.iRows];    // Data record

      for (int i=0; i<stIn.iRows; i++)
      {
        stOut.ts[i]  = new Timestamp(stIn.ts[i].getTime());

        p = p + q;

        K = p / ( p + r );
        x = x + K * ( stIn.data[i] - x );
        p = (1 - K) * p;

		stOut.data[i] = x;
	  }
	}

	return (stOut);
  }

}

/*
 *  @version 1.1 05/18/2016
 *  @author  John Victorine
 */
