/*
 * @seisControlPanel.java Version 1.1 05/11/2016
 *
 * Copyright (c) 2016 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package seismic.gui;

import java.util.Observable;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.sql.Timestamp;
import java.sql.Date;
import java.text.SimpleDateFormat;
import java.text.ParseException;

import cmn.cmnStruct;                  // Global Common Data Structure

import seismic.seisStruct;             // Seismic Data Structure
import seismic.gui.seisFrequencyFrame; // Compute Frequency Frame
import seismic.gui.seisSmoothingPanel; // Smooth Seismic Wave Signal
import seismic.gui.seisPlotFrame;      // Seismic Plot Frame

/** Class seisControlPanel
 *  <p> This Class is the Plot Control Frame for plot frame.
 *
 *  @version 1.1 05/11/2016
 *  @author  John R. Victorine
 */

public class seisControlPanel extends JPanel implements ActionListener
{
  private static final int _SEISMIC       = seismic.seisTracksStruct._SEISMIC;
  private static final int _SEISMIC_AVG   = seismic.seisTracksStruct._SEISMIC_AVG;
  private static final int _SEISMIC_MAG   = seismic.seisTracksStruct._SEISMIC_MAG;

  // Input Variables

  private cmnStruct          stCMN        = null; // Common Data Structure
  private Observable         notifier     = null; // Observables Object
  private seisStruct         stData       = null; // Raw Seismic Data

  // Global Variables

  private seisSmoothingPanel pnlWave      = null; // Constructed Wave
  private seisFrequencyFrame pnlFrequency = null; // Frequency Space Calculator
  private seisPlotFrame      plotframe    = null; // Seismic Data Plot Frame

  private static final int MINIMUM   = 2;
  private static final int MAXIMUM   = 3;

  int iData = -1;

  // Global Frame Widgets

  // -- Header File Data

  private JLabel lblNetwork      = new JLabel(); // Network Code
  private JLabel lblStation      = new JLabel(); // Station Identifier
  private JLabel lblLocation     = new JLabel(); // Location Identifier
  private JLabel lblChannel      = new JLabel(); // Channel Identifier

  // -- Track 1 Minimum & Maximum Limits Widgets

  private JTextField txtMinimum1 = new JTextField(); // Minimum Value
  private JTextField txtMaximum1 = new JTextField(); // Maximum Value

  // -- Frequency Plot Button

  private JButton    btnFilter   = new JButton();    // Filter Data Button
  private JButton    btnFreq     = new JButton();    // Frequency Button

  /** Constructor seisControlPanel( cmnStruct stCMN )
   *  <p> This is the Constructor for this class.
   *  @param stCMN = Global IQSTRAT Data Structure
   */

  public seisControlPanel( Observable notifier, cmnStruct stCMN, seisStruct stData )
  {
    try
    {
      this.notifier = notifier;
      this.stCMN    = stCMN;
      this.stData   = stData;

      jbInit();

      if (stData != null)
      {
        lblNetwork.setText("Network:  "  + stData.network);  // Network Code
        lblStation.setText("Station:  "  + stData.station);  // Station Identifier
        lblLocation.setText("Location: " + stData.location); // Location Identifier
        lblChannel.setText("Channel:  "  + stData.channel);  // Channel Identifier

        txtMinimum1.setText("" + stData.dMinimum);
        txtMaximum1.setText("" + stData.dMaximum);
      }
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Panel for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
    JPanel pnlTop            = new JPanel();
    JPanel pnlHeader         = new JPanel();

    JPanel pnlCenter         = new JPanel();
    JPanel pnlTrack1         = new JPanel();

    JPanel pnl               = new JPanel();
    JPanel pnl2              = new JPanel();

    JPanel pnlFreq           = new JPanel();

    // Build Panel

    this.setLayout(new BorderLayout());

    // Top Panel

    pnlTop.setLayout(new BorderLayout());

    // -- Header Panel

    pnlHeader   = BuildHeaderPanel();

    // Panel Center

    pnlCenter.setLayout(new BorderLayout());

    // -- Track 1 (Seismic Wave) Panel

    pnlTrack1 = BuildTrack1LimitsPanel();

    // Convolve Panel

    pnl.setLayout(new BorderLayout());

    pnlWave = new seisSmoothingPanel( stCMN, notifier );
    pnlWave.setData( stData );

    // Frequency Button Panel

    pnlFreq.setLayout(new GridLayout());

    btnFilter.setFont(new java.awt.Font("Dialog", 1, 11));
    btnFilter.setText("Filter Raw Data");
    btnFilter.addActionListener(this);

    btnFreq.setFont(new java.awt.Font("Dialog", 1, 11));
    btnFreq.setText("Create Frequency Plot");
    btnFreq.addActionListener(this);

    // Frequency Panel

    pnl2.setLayout(new BorderLayout());

    // Add Widgets to Frame

//    this.add(pnlTop,         BorderLayout.NORTH);
    pnlTop.add(pnlHeader,    BorderLayout.NORTH);

    this.add(pnlCenter,      BorderLayout.CENTER);
    pnlCenter.add(pnlTrack1, BorderLayout.NORTH);

    pnlCenter.add(pnl,       BorderLayout.CENTER);
    pnl.add(pnlWave,         BorderLayout.NORTH);

    pnl.add(pnl2,            BorderLayout.CENTER);
    pnl2.add(pnlFreq,        BorderLayout.NORTH);
    pnlFreq.add(btnFilter,   null);
    pnlFreq.add(btnFreq,     BorderLayout.CENTER);
  }

  /** Method BuildHeaderPanel()
   *  <p> This method will build the header panel on the control dialog
   */

  private JPanel BuildHeaderPanel()
  {
    JPanel pnlHeader = new JPanel();

    TitledBorder titledBorderHeader = new TitledBorder(
        new EtchedBorder(EtchedBorder.RAISED,
                         Color.white,
                         new Color(165, 163, 151)),
        "Header Information");
    titledBorderHeader.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    // -- Header Panel

    pnlHeader.setLayout(new GridLayout(4,1));
    pnlHeader.setBorder(titledBorderHeader);

    lblNetwork.setText("Network:  ");  // Network Code
    lblNetwork.setFont(new java.awt.Font("monospaced", 1, 12));

    lblStation.setText("Station:  ");  // Station Identifier
    lblStation.setFont(new java.awt.Font("monospaced", 1, 12));

    lblLocation.setText("Location: "); // Location Identifier
    lblLocation.setFont(new java.awt.Font("monospaced", 1, 12));

    lblChannel.setText("Channel:  ");  // Channel Identifier
    lblChannel.setFont(new java.awt.Font("monospaced", 1, 12));

    pnlHeader.add(lblNetwork,  null);
    pnlHeader.add(lblStation,  null);
    pnlHeader.add(lblLocation, null);
    pnlHeader.add(lblChannel,  null);

    return (pnlHeader);
  }

  /** Method BuildTrack1LimitsPanel()
   *  <p> this method will build the Track 1 Data Limits Panel
   */

  private JPanel BuildTrack1LimitsPanel()
  {
    JPanel pnlTrack         = new JPanel();
    JPanel pnlMinimum       = new JPanel();
    JPanel pnlMaximum       = new JPanel();

    TitledBorder titledBorderWaves = new TitledBorder(
        new EtchedBorder(EtchedBorder.RAISED,
                         Color.white,
                         new Color(165, 163, 151)),
        seismic.seisTracksStruct.SEIS_TRACK_DESC[0]+" Plot Track");
    titledBorderWaves.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    TitledBorder titledBorderMinimum = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Minimum");
    titledBorderMinimum.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderMaximum = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Maximum");
    titledBorderMaximum.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    // -- Track 1 (Seismic Wave) Panel

    pnlTrack.setLayout(new GridLayout());
    pnlTrack.setBorder(titledBorderWaves);

    pnlMinimum.setLayout(new BorderLayout());
    pnlMinimum.setBorder(titledBorderMinimum);
    pnlMinimum.add(txtMinimum1);

    txtMinimum1.setText("");
	txtMinimum1.setHorizontalAlignment(SwingConstants.TRAILING);
	txtMinimum1.addFocusListener(new seisControlPanelFocusAdapter(this));

    pnlMaximum.setLayout(new BorderLayout());
    pnlMaximum.setBorder(titledBorderMaximum);
    pnlMaximum.add(txtMaximum1);

    txtMaximum1.setText("");
	txtMaximum1.setHorizontalAlignment(SwingConstants.TRAILING);
	txtMaximum1.addFocusListener(new seisControlPanelFocusAdapter(this));

    pnlTrack.add(pnlMinimum, null);
    pnlTrack.add(pnlMaximum, null);

    return (pnlTrack);
  }

  /* ===================================================================== *
   * ----------------------------- CLOSE --------------------------------- *
   * ===================================================================== */

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    // Input Variables

    stCMN     = null; // Global IQSTRAT Data Structure
    notifier  = null; // Observables Object
    stData    = null; // Seismic Data Structure

    // Global Variables

    if (pnlWave != null)
      pnlWave.close();
    pnlWave   = null; // Construct Wave Panel

    if (pnlFrequency != null)
      pnlFrequency.close();
    pnlFrequency = null; // Convolve Panel

    plotframe = null; // Seismic Data Plot Frame

    // Global Frame Widgets

    // -- Header File Data

    lblNetwork  = null; // Network Code
    lblStation  = null; // Station Identifier
    lblLocation = null; // Location Identifier
    lblChannel  = null; // Channel Identifier

    // -- Track 1 Minimum & Maximum Limits Widgets

    txtMinimum1 = null; // Minimum Value
    txtMaximum1 = null; // Maximum Value

    // -- Frequency Plot Button

    btnFilter   = null; // Filter Data Button
    btnFreq     = null; // Frequecy Plot Button
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getAvgData()
   *  <p> This method will retrieve the Averaged Data Set
   *  @return stAvg = the averaged seismic data structure
   */

  public seisStruct getAvgData() { return (pnlWave.getData()); }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setPlot()
   *  <p> This method will pass the plotframe pointer
   *  @plotframe = the plot frame pointer
   */

  public void setPlot(seisPlotFrame plotframe)
  {
	this.plotframe = plotframe;
    pnlWave.setPlot( plotframe );
  }

  /** Method shareData()
   *  <p> This method will share the Average Data
   */

  public void shareData()
  {
    if ((pnlFrequency != null) && (pnlWave != null))
	{
	  pnlFrequency.setTitle(pnlWave.getTitle());
	  pnlFrequency.setAvgData(pnlWave.getData());
	}
  }

  /* ===================================================================== *
   * ------------------ FREQUENCY CALCULATOR ACTION ---------------------- *
   * ===================================================================== */

  /** Method displayFrequency()
   *  <p> This method will display the frequency calculator frame
   */

  private void displayFrequency()
  {
	if (pnlFrequency != null)
	  pnlFrequency.close();
	pnlFrequency = null;

    pnlFrequency = new seisFrequencyFrame( stCMN, notifier );
    pnlFrequency.setData( stData );

    if (pnlWave != null)
    {
   	  pnlFrequency.setTitle(pnlWave.getTitle());
	  pnlFrequency.setAvgData(pnlWave.getData());
	}
  }

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
	if (event.getSource() == btnFilter) { pnlWave.displayFilterFrame(); }
    if (event.getSource() == btnFreq) { displayFrequency();	}
  }

  /** METHOD focusLost()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLost(FocusEvent e)
  {
	focusLostTrack1(e);
  }

  /** METHOD focusLostTrack1()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLostTrack1(FocusEvent e)
  {
    int     iNumeric = 0;
    int     len      = 0;
    int     iField   = -1;
    boolean bVal     = true;
    String  sMessage = new String("");
    String  sTemp    = new String("");
    int     id       = 0;
    double  d        = 0.0;
    double  d1       = 0.0;

    if (e.getSource() == txtMinimum1)
    {
      iData    = MINIMUM;
      iNumeric = 1;
      sTemp    = new String( txtMinimum1.getText().trim());
      sMessage = new String("Minimum Value is a Numeric Field");
    }

    if (e.getSource() == txtMaximum1)
    {
      iData    = MAXIMUM;
      iNumeric = 1;
      sTemp    = new String( txtMaximum1.getText().trim());
      sMessage = new String("Maximum Value is a Numeric Field");
    }

    if (iNumeric == 1)
    {
      if (!cmn.cmnString.isNumeric(sTemp))
      {
        JOptionPane.showMessageDialog((Component) null,
                                       sMessage,
                                       "ERROR",
                                       JOptionPane.ERROR_MESSAGE);

        switch (iData)
       {
          case MINIMUM:
            txtMinimum1.setText(""+stData.dMinimum);
            break;

          case MAXIMUM:
            txtMaximum1.setText(""+stData.dMaximum);
            break;
	    }
      }
      else
      {
        switch (iData)
        {
          case MINIMUM:
          case MAXIMUM:
            d  = cmn.cmnString.stringToDouble(txtMinimum1.getText());
            d1 = cmn.cmnString.stringToDouble(txtMaximum1.getText());
            plotframe.setLimits( _SEISMIC, d, d1 );
            break;
	    }
      }
    }
  }

}

/** CLASS seisControlPanelFocusAdapter()
 *  <p> This Class will handle Actions when focus is lost on a textfield
 */

class seisControlPanelFocusAdapter extends java.awt.event.FocusAdapter
{
  seisControlPanel adaptee;

  seisControlPanelFocusAdapter(seisControlPanel adaptee)
  {
    this.adaptee = adaptee;
  }

  public void focusLost(FocusEvent e) { adaptee.focusLost(e); }
}

/*
 *  @version 1.1 05/11/2016
 *  @author  John Victorine
 */
