/*
 * @seisPlotPanel.java Version 1.1 03/05/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package seismic.gui;

import java.util.Observable;
import java.sql.Timestamp;
import java.awt.*;
import java.awt.image.*;
import javax.swing.*;

import cmn.cmnStruct;             // Global Common Data Structure
import seismic.seisStruct;        // Seismic Data Structure
import seismic.plot.seisPlot;     // Seismic Data Plot

/** Class seisPlotPanel
 *  <p> This Class will create the stratigraphic Plot.
 *
 *  @version 1.1 03/05/2015
 *  @author  John R. Victorine
 */

public class seisPlotPanel extends JPanel
{
  // Global Variables

  private int iWidth   = 0;
  private int iHeight  = 0;

  private seisPlot plot   = null;

  // Panel Widgets

  private ScrollPane scroll = new ScrollPane();
  private Panel      panel  = new Panel();

  /** CONSTRUCTOR seisPlot()
   *  <p> This is the constructor for this class.
   *  @param stCMN        = Common Data Structure
   *  @param tsStart      = The starting date
   *  @param tsEnd        = The ending date
   *  @param iScale       = The time scale
   *  @param iSelected    = The selected tracks for plotting
   *  @param normalCursor = Normal Cursor
   *  @param mouseCursor  = Hand Cursor
   */

  public seisPlotPanel( cmnStruct stCMN,
                        Timestamp tsStart,
                        Timestamp tsEnd,
                        int       iScale,
                        int       iSelected[],
                        Cursor    normalCursor,
                        Cursor    mouseCursor )
  {
    try
    {
      jbInit(stCMN, tsStart, tsEnd, iScale, iSelected, normalCursor, mouseCursor);
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Panel for this class.
   *  @param stCMN        = Common Data Structure
   *  @param tsStart      = The starting date
   *  @param tsEnd        = The ending date
   *  @param iScale       = The time scale
   *  @param iSelected    = The selected tracks for plotting
   *  @param normalCursor = Normal Cursor
   *  @param mouseCursor  = Hand Cursor
   *  @throws Exception
   */

  private void jbInit( cmnStruct stCMN,
                       Timestamp tsStart,
                       Timestamp tsEnd,
                       int       iScale,
                       int       iSelected[],
                       Cursor    normalCursor,
                       Cursor    mouseCursor ) throws Exception
  {
    Panel pnlCenter = new Panel();

    this.setLayout(new BorderLayout());
    pnlCenter.setLayout(new BorderLayout());

    plot    = new seisPlot( stCMN, tsStart, tsEnd, iScale, iSelected,
                            normalCursor, mouseCursor);
    iWidth  = 50 + plot.getPlotWidth();
    iHeight = plot.getPlotHeight();

    panel.setLayout( new BorderLayout() );
    panel.setPreferredSize( new Dimension( iWidth, iHeight ) );

    this.add(pnlCenter,   BorderLayout.CENTER);
    pnlCenter.add(scroll, BorderLayout.CENTER);
    scroll.add(panel,     null);
    panel.add(plot,       BorderLayout.CENTER);
  }


  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    scroll  = null;
    panel   = null;

    if (plot != null)
      plot.close();
    plot    = null;
  }

  /** Method refresh()
   *  <p> This method will refresh the plot
   */

  public void refresh() { if (plot != null) { plot.refresh(); } }

  /* ======================================================================= *
   * ----------------------------- GET METHODS ----------------------------- *
   * ======================================================================= */

  /** Method getPlotWidth()
   * <p> This method will return the Plot Width
   * @return iWidth = The Plot Width
   */

  public int  getPlotWidth()  { return (iWidth); }

  /** Method getPlotHeight()
   * <p> This method will return the Plot Height
   * @return iHeight = The Plot Height
   */

  public int getPlotHeight() { return (iHeight); }


  /** Method getImage()
   * <p> This method will convert this plot to a buffered image
   * @return image = profile plot buffered image
   */

  public BufferedImage getImage()
  {
    BufferedImage image = null;

    if (plot != null) image = plot.getImage();

    return (image);
  }

  /* ======================================================================= *
   * ----------------------------- SET METHODS ----------------------------- *
   * ======================================================================= */

  /** Method setObservable( Observable notifier )
   * <p> This method will set the Observerable Object
   * @param notifier = the observable object
   */

  public void setObservable( Observable notifier )
  {
    if (plot != null) plot.setObservable( notifier );
  }

  /** Method setTitles( String sTitle1, String sTitle2 )
   * <p> This method will set the titles for this plot
   * @param sTitle1 = The first title
   * @param sTitle2 = The second title
   */

  public void setTitles( String sTitle1, String sTitle2 )
  {
    if (plot != null)
    {
      plot.setTitles(sTitle1, sTitle2);
    }
  }

  /** Method setScale(int iScale, double dMaximum, double dMinimum)
   * <p> This method will set the Plot scale in the
   *     Cross Plot Data List Structure
   * @param iScale    = The Time Track Scale identifier
   * @param tsMaximum = The Maximum Timestamp
   * @param tsMinimum = The Minimum Timestamp
   */

  public void setScale(int iScale, Timestamp tsMaximum, Timestamp tsMinimum)
  {
    if (plot != null)
    {
      plot.setScale( iScale, tsMaximum, tsMinimum );
      iWidth  = 50 + plot.getPlotWidth();
      iHeight = plot.getPlotHeight();
      panel.setPreferredSize( new Dimension( iWidth, iHeight ) );
    }

    scroll.doLayout();
    panel.doLayout();
  }

  /* ----------------------------------------------------------------------- *
   * ------------------- Mouse Time Location Textfield -------------------- *
   * ----------------------------------------------------------------------- */

  /** Method setTextfield( JTextField txt )
   * <p> This method will set the time textfield
   * @param txt = The Time Text Field
   */

  public void setTextfield( JTextField txt )
  {
    if (plot != null) plot.setTextfield (txt );
  }

  /* ----------------------------------------------------------------------- *
   * ----------------- Available Tracks Selected for view ------------------ *
   * ----------------------------------------------------------------------- */

  /** Method setSelected( int iSelected[] )
   * <p> This method will change the selected tracks
   *  @param iSelected  = The Tracks that will be displayed
   */

  public void setSelected( int iSelected[] )
  {
    if (plot != null) plot.setSelected( iSelected );
  }

  /** Method setData( seisStruct st )
   * <p> This method will set the seismic Data Structure
   * @param iData - Data being passed
   * @param st - seismic Data Structure
   */

  public void setLimits(int iData, double dMinimum, double dMaximum)
  {
	if (plot != null) { plot.setLimits( iData, dMinimum, dMaximum); }
  }

  /** Method setData( int iData, seisStruct st )
   * <p> This method will set the Seismic Data Structure
   * @param iData - Data being passed
   * @param st    - Seismic Data Structure
   */

  public void setData( int iData, seisStruct st )
  {
    if (plot != null) plot.setData( iData, st );
  }
}

/*
 *  @version 1.1 03/05/2015
 *  @author  John Victorine
 */
